@@ -36,12 +36,12 @@
 #include <errno.h>
 
 #include <libxfce4util/libxfce4util.h>
-#include <libxfcegui4/libxfcegui4.h>
 
 #include <libburn.h>
 
 #include <gst/gst.h>
 #include <gst/pbutils/missing-plugins.h>
+#include <gst/pbutils/pbutils.h>
 
 #include "xfburn-global.h"
 #include "xfburn-error.h"
@@ -64,8 +64,6 @@
 
 /** Prototypes **/
 /* class initialization */
-static void xfburn_transcoder_gst_class_init (XfburnTranscoderGstClass * klass);
-static void xfburn_transcoder_gst_init (XfburnTranscoderGst * obj);
 static void xfburn_transcoder_gst_finalize (GObject * object);
 static void transcoder_interface_init (XfburnTranscoderInterface *iface, gpointer iface_data);
 
@@ -85,18 +83,15 @@ static void finish (XfburnTranscoder *trans);
 
 static gboolean is_initialized (XfburnTranscoder *trans, GError **error);
 
-static gboolean signal_identification_done (XfburnTranscoderGst *trans, const char *dbg_res);
-static gboolean query_and_signal_duration (XfburnTranscoderGst *trans);
-
 /* gstreamer support functions */
 static gboolean bus_call (GstBus *bus, GstMessage *msg, gpointer data);
-static void on_pad_added (GstElement *element, GstPad *pad, gboolean last, gpointer data);
+static void on_pad_added (GstElement *element, GstPad *pad, gpointer data);
 
 #ifdef DEBUG_GST
 static void cb_handoff (GstElement *element, GstBuffer *buffer, gpointer data);
 #endif
 
-#define XFBURN_TRANSCODER_GST_GET_PRIVATE(obj) (G_TYPE_INSTANCE_GET_PRIVATE ((obj), XFBURN_TYPE_TRANSCODER_GST, XfburnTranscoderGstPrivate))
+#define XFBURN_TRANSCODER_GST_GET_PRIVATE(obj) (xfburn_transcoder_gst_get_instance_private ( XFBURN_TRANSCODER_GST (obj)))
 
 enum {
   LAST_SIGNAL,
@@ -104,21 +99,22 @@ enum {
 
 typedef enum {
   XFBURN_TRANSCODER_GST_STATE_IDLE,
-  XFBURN_TRANSCODER_GST_STATE_IDENTIFYING,
   XFBURN_TRANSCODER_GST_STATE_TRANSCODE_START,
   XFBURN_TRANSCODER_GST_STATE_TRANSCODING,
 } XfburnTranscoderGstState;
 
 typedef struct {
   GstElement *pipeline;
-  GstElement *source, *decoder, *resample, *conv, *sink;
+  GstElement *source, *decoder, *resample, *conv1, *conv2, *sink;
 
   XfburnTranscoderGstState state;
-  GCond *gst_cond;
-  GMutex *gst_mutex;
-  gboolean is_audio;
+  GCond gst_cond;
+  GMutex gst_mutex;
+  gboolean gst_done;
   gint64 duration;
 
+  GstDiscoverer *discoverer;
+
   GError *error;
 
   GSList *tracks;
@@ -129,7 +125,7 @@ typedef struct {
 
 /* constants */
 
-#define SIGNAL_WAIT_TIMEOUT_MICROS 1500000
+#define SIGNAL_WAIT_TIMEOUT_MS 1500
 
 #define SIGNAL_SEND_ITERATIONS 10
 /* SIGNAL_SEND_TIMEOUT_MICROS is the total time,
@@ -145,8 +141,13 @@ typedef struct {
 static guint64 total_size = 0;
 #endif
 
-static const gchar *errormsg_gst_setup = "An error occurred setting gstreamer up for transcoding";
-static const gchar *errormsg_libburn_setup = "An error occurred while setting the burning backend up";
+static const gchar *errormsg_gst_setup = N_("An error occurred setting gstreamer up for transcoding");
+static const gchar *errormsg_libburn_setup = N_("An error occurred while setting the burning backend up");
+static const gchar *errormsg_missing_plugin = N_("%s is missing.\n"
+                                              "\n"
+                                              "You do not have a decoder installed to handle this file.\n"
+                                              "Probably you need to look at the gst-plugins-* packages\n"
+                                              "for the necessary plugins.\n");
 
 /*********************/
 /* class declaration */
@@ -154,45 +155,20 @@ static const gchar *errormsg_libburn_setup = "An error occurred while setting th
 static GObject *parent_class = NULL;
 //static guint signals[LAST_SIGNAL];
 
-GType
-xfburn_transcoder_gst_get_type (void)
-{
-  static GType type = 0;
-
-  if (type == 0) {
-    static const GTypeInfo our_info = {
-      sizeof (XfburnTranscoderGstClass),
-      NULL,
-      NULL,
-      (GClassInitFunc) xfburn_transcoder_gst_class_init,
-      NULL,
-      NULL,
-      sizeof (XfburnTranscoderGst),
-      0,
-      (GInstanceInitFunc) xfburn_transcoder_gst_init,
-      NULL
-    };
-    static const GInterfaceInfo trans_info = {
-      (GInterfaceInitFunc) transcoder_interface_init,
-      NULL,
-      NULL
-    };
-
-    type = g_type_register_static (G_TYPE_OBJECT, "XfburnTranscoderGst", &our_info, 0);
-
-    g_type_add_interface_static (type, XFBURN_TYPE_TRANSCODER, &trans_info);
-  }
-
-  return type;
-}
+G_DEFINE_TYPE_EXTENDED(
+  XfburnTranscoderGst,
+  xfburn_transcoder_gst,
+  G_TYPE_OBJECT,
+  0,
+  G_ADD_PRIVATE(XfburnTranscoderGst)
+  G_IMPLEMENT_INTERFACE(XFBURN_TYPE_TRANSCODER, transcoder_interface_init)
+);
 
 static void
 xfburn_transcoder_gst_class_init (XfburnTranscoderGstClass * klass)
 {
   GObjectClass *object_class = G_OBJECT_CLASS (klass);
   
-  g_type_class_add_private (klass, sizeof (XfburnTranscoderGstPrivate));
-  
   parent_class = g_type_class_peek_parent (klass);
 
   object_class->finalize = xfburn_transcoder_gst_finalize;
@@ -214,12 +190,14 @@ xfburn_transcoder_gst_init (XfburnTranscoderGst * obj)
 
   /* the condition is used to signal that
    * gst has returned information */
-  priv->gst_cond = g_cond_new ();
+  g_cond_init (&priv->gst_cond);
 
   /* if the mutex is locked, then we're not currently seeking
    * information from gst */
-  priv->gst_mutex = g_mutex_new ();
-  g_mutex_lock (priv->gst_mutex);
+  g_mutex_init (&priv->gst_mutex);
+  /* Actual lock acquired in prepare() and relased in finish() */
+
+  priv->discoverer = gst_discoverer_new(GST_SECOND, NULL);
 }
 
 static void
@@ -230,6 +208,7 @@ xfburn_transcoder_gst_finalize (GObject * object)
   gst_element_set_state (priv->pipeline, GST_STATE_NULL);
 
   gst_object_unref (GST_OBJECT (priv->pipeline));
+  g_object_unref (G_OBJECT (priv->discoverer));
   priv->pipeline = NULL;
 
   G_OBJECT_CLASS (parent_class)->finalize (object);
@@ -257,7 +236,7 @@ create_pipeline (XfburnTranscoderGst *trans)
 {
   XfburnTranscoderGstPrivate *priv= XFBURN_TRANSCODER_GST_GET_PRIVATE (trans);
 
-  GstElement *pipeline, *source, *decoder, *resample, *conv, *sink;
+  GstElement *pipeline, *source, *decoder, *resample, *conv1, *conv2, *sink;
 #if DEBUG_GST > 0 && DEBUG > 0
   GstElement *id;
 #endif
@@ -268,18 +247,18 @@ create_pipeline (XfburnTranscoderGst *trans)
 
   priv->pipeline = pipeline = gst_pipeline_new ("transcoder");
 
-  priv->source  = source   = gst_element_factory_make ("filesrc",       "file-source");
-  priv->decoder = decoder  = gst_element_factory_make ("decodebin",     "decoder");
-  priv->resample= resample = gst_element_factory_make ("audioresample", "resampler");
-  priv->conv    = conv     = gst_element_factory_make ("audioconvert",  "converter");
+  priv->source   = source   = gst_element_factory_make ("filesrc",       "file-source");
+  priv->decoder  = decoder  = gst_element_factory_make ("decodebin",     "decoder");
+  priv->conv1    = conv1    = gst_element_factory_make ("audioconvert",  "converter1");
+  priv->resample = resample = gst_element_factory_make ("audioresample", "resampler");
+  priv->conv2    = conv2    = gst_element_factory_make ("audioconvert",  "converter2");
 #if DEBUG_GST > 0 && DEBUG > 0
                   id       = gst_element_factory_make ("identity",      "debugging-identity");
 #endif
   priv->sink    = sink     = gst_element_factory_make ("fdsink",        "audio-output");
   //priv->sink    = sink     = gst_element_factory_make ("fakesink",        "audio-output");
-  //DBG ("\npipeline = %p\nsource = %p\ndecoder = %p\nconv = %p\nsink = %p", pipeline, source, decoder, conv, sink);
 
-  if (!pipeline || !source || !decoder || !resample || !conv || !sink) {
+  if (!pipeline || !source || !decoder || !resample || !conv1 || !conv2 || !sink) {
     g_warning ("A pipeline element could not be created");
     g_set_error (&(priv->error), XFBURN_ERROR, XFBURN_ERROR_GST_CREATION,
 		    "%s",
@@ -304,28 +283,27 @@ create_pipeline (XfburnTranscoderGst *trans)
   gst_object_unref (bus);
 
   gst_bin_add_many (GST_BIN (pipeline),
-                    source, decoder, resample, conv, sink, NULL);
+                    source, decoder, conv1, resample, conv2, sink, NULL);
 #if DEBUG_GST > 0 && DEBUG > 0
   gst_bin_add (GST_BIN (pipeline), id);
 #endif
 
   gst_element_link (source, decoder);
-  gst_element_link (resample, conv);
+  gst_element_link (conv1, resample);
+  gst_element_link (resample, conv2);
 
   /* setup caps for raw pcm data */
-  caps = gst_caps_new_simple ("audio/x-raw-int",
+  caps = gst_caps_new_simple ("audio/x-raw",
             "rate", G_TYPE_INT, 44100,
             "channels", G_TYPE_INT, 2,
-            "endianness", G_TYPE_INT, G_LITTLE_ENDIAN,
-            "width", G_TYPE_INT, 16,
-            "depth", G_TYPE_INT, 16,
-            "signed", G_TYPE_BOOLEAN, TRUE,
+            "format", G_TYPE_STRING, "S16LE",
             NULL);
 
 #if DEBUG_GST > 0 && DEBUG > 0
-  if (!gst_element_link_filtered (conv, id, caps)) {
+  // TODO without the filter it does work but I'm not sure the audio format is correct
+  if (!gst_element_link_filtered (conv2, id, caps)) {
 #else
-  if (!gst_element_link_filtered (conv, sink, caps)) {
+  if (!gst_element_link_filtered (conv2, sink, caps)) {
 #endif
     g_warning ("Could not setup filtered gstreamer link");
     g_set_error (&(priv->error), XFBURN_ERROR, XFBURN_ERROR_GST_CREATION,
@@ -340,7 +318,7 @@ create_pipeline (XfburnTranscoderGst *trans)
   g_signal_connect (id, "handoff", G_CALLBACK (cb_handoff), id);
 #endif
 
-  g_signal_connect (decoder, "new-decoded-pad", G_CALLBACK (on_pad_added), trans);
+  g_signal_connect (decoder, "pad-added", G_CALLBACK (on_pad_added), trans);
 }
 
 static void 
@@ -389,80 +367,6 @@ state_to_str (GstState st)
 }
 #endif
 
-/*
- * this function expects the result to be in priv->is_audio, 
- * dbg_res is only for debugging output
- */
-static gboolean
-signal_identification_done (XfburnTranscoderGst *trans, const char *dbg_res)
-{
-  XfburnTranscoderGstPrivate *priv= XFBURN_TRANSCODER_GST_GET_PRIVATE (trans);
-
-  int i;
-
-#if DEBUG_GST > 0
-  DBG ("Trying to lock mutex (%s)", dbg_res);
-#endif
-
-  /* There is no g_mutex_lock_timed, so emulate it with a loop.
-    * I have never seen this getting hung here, but one never knows! */
-  for (i=0; i<SIGNAL_SEND_ITERATIONS; i++) {
-    if (g_mutex_trylock (priv->gst_mutex))
-      break;
-    g_usleep (SIGNAL_SEND_TIMEOUT_MICROS / SIGNAL_SEND_ITERATIONS);
-    g_thread_yield ();
-    if (i==9) {
-      g_warning ("Noone was there to listen to the result of the identification!");
-      /* FIXME: recreate pipeline here? This state is not the fault of gst */
-      if (gst_element_set_state (priv->pipeline, GST_STATE_NULL) == GST_STATE_CHANGE_FAILURE) {
-        DBG ("Oops, could not reset pipeline to null");
-      }
-      return FALSE;
-    }
-  }
-
-  g_cond_signal (priv->gst_cond);
-  g_mutex_unlock (priv->gst_mutex);
-
-#if DEBUG_GST > 0
- #if DEBUG > 0
-  DBG ("Releasing mutex (%s)", dbg_res);
- #else
-  g_message ("Signaled identification done: %s", dbg_res);
- #endif
-#endif
-
-  return TRUE;
-}
-
-
-static gboolean
-query_and_signal_duration (XfburnTranscoderGst *trans)
-{
-  XfburnTranscoderGstPrivate *priv= XFBURN_TRANSCODER_GST_GET_PRIVATE (trans);
-  GstFormat fmt;
-  //guint secs;
-
-  fmt = GST_FORMAT_TIME;
-  if (!gst_element_query_duration (priv->pipeline, &fmt, &priv->duration)) {
-    return FALSE;
-  }
-
-  //secs = priv->duration / 1000000000;
-  if (gst_element_set_state (priv->pipeline, GST_STATE_NULL) == GST_STATE_CHANGE_FAILURE) {
-    DBG ("Failed to set state!");
-    recreate_pipeline (trans);
-  }
-
-  priv->is_audio = TRUE;
-  priv->state = XFBURN_TRANSCODER_GST_STATE_IDLE;
-
-  signal_identification_done (trans, "is audio");
-
-  return TRUE;
-}
-
-
 static gboolean
 bus_call (GstBus *bus, GstMessage *msg, gpointer data)
 {
@@ -518,16 +422,6 @@ bus_call (GstBus *bus, GstMessage *msg, gpointer data)
           recreate_pipeline (trans);
           break;
 
-        case XFBURN_TRANSCODER_GST_STATE_IDENTIFYING:
-          recreate_pipeline (trans);
-
-          g_message ("gstreamer abort: %s\n", error->message);
-          priv->is_audio = FALSE;
-          priv->error = error;
-
-          signal_identification_done (trans, "error");
-          break;
-
         case XFBURN_TRANSCODER_GST_STATE_TRANSCODE_START:
         case XFBURN_TRANSCODER_GST_STATE_TRANSCODING:
           g_error ("Gstreamer error while transcoding: %s", error->message);
@@ -542,7 +436,7 @@ bus_call (GstBus *bus, GstMessage *msg, gpointer data)
         DBG ("Failed to reset GST state.");
         recreate_pipeline (trans);
       }
-      signal_identification_done (trans, "not audio content");
+      DBG("GST recognized an application, ignoring.");
 
       break;
     }
@@ -565,43 +459,22 @@ bus_call (GstBus *bus, GstMessage *msg, gpointer data)
         case XFBURN_TRANSCODER_GST_STATE_TRANSCODING:
           break;
 
-        case XFBURN_TRANSCODER_GST_STATE_IDENTIFYING:
-          if (state != GST_STATE_PAUSED)
-            break;
-          if (!query_and_signal_duration (trans)) {
-            /* this is expected to fail a couple of times,
-             * so no output unless we're in debug mode */
-#if DEBUG_GST > 1
-            DBG ("could not query stream duration (expected failure)");
-#endif
-          }
-          break;
-
         case XFBURN_TRANSCODER_GST_STATE_TRANSCODE_START:
           if (state != GST_STATE_PLAYING)
             break;
 
           if (strcmp (GST_OBJECT_NAME (GST_MESSAGE_SRC (msg)), "decoder") != 0)
             break;
-          
-          if (!g_mutex_trylock (priv->gst_mutex)) {
-            g_critical ("Lock held by another thread, can't signal transcoding start!");
-            break;
-          } else {
-#if DEBUG_GST > 0
-            DBG ("Locked mutex to signal transcoding start");
-#endif
-          }
 
-          g_cond_signal (priv->gst_cond);
-          g_mutex_unlock (priv->gst_mutex);
+          priv->gst_done = TRUE;
+          g_cond_signal (&priv->gst_cond);
           break;
       } /* switch of priv->state */
 
       break;
     }
 
-    case GST_MESSAGE_DURATION: {
+    case GST_MESSAGE_DURATION_CHANGED: {
 
       switch (priv->state) {
         case XFBURN_TRANSCODER_GST_STATE_IDLE:
@@ -609,34 +482,20 @@ bus_call (GstBus *bus, GstMessage *msg, gpointer data)
         case XFBURN_TRANSCODER_GST_STATE_TRANSCODE_START:
           break;
 
-        case XFBURN_TRANSCODER_GST_STATE_IDENTIFYING:
-          if (!query_and_signal_duration (trans)) {
-            /* this is expected to work, because we got the duration message on the bus */
-#if DEBUG_GST > 0
-            g_warning ("Could not query stream length!");
-#endif
-            return TRUE;
-          }
-          break;
       } /* switch of priv->state */
 
       break;
     }
 
     case GST_MESSAGE_ELEMENT: {
+      /* This should not happen now that the discoverer API is used to identify songs.
+       * Shouldn't hurt to keep around though. */
       if (gst_is_missing_plugin_message (msg)) {
           recreate_pipeline (trans);
 
-          priv->is_audio = FALSE;
           g_set_error (&(priv->error), XFBURN_ERROR, XFBURN_ERROR_MISSING_PLUGIN,
-                       _("%s is missing.\n"
-                         "\n"
-                         "You do not have a decoder installed to handle this file.\n"
-                         "Probably you need to look at the gst-plugins-* packages\n"
-                         "for the necessary plugins.\n"),
+                       _(errormsg_missing_plugin),
                         gst_missing_plugin_message_get_description (msg)); 
-
-          signal_identification_done (trans, "missing-plugin");
       }
     }
     default:
@@ -650,7 +509,7 @@ bus_call (GstBus *bus, GstMessage *msg, gpointer data)
 }
 
 static void
-on_pad_added (GstElement *element, GstPad *pad, gboolean last, gpointer data)
+on_pad_added (GstElement *element, GstPad *pad, gpointer data)
 {
   XfburnTranscoderGst *trans = XFBURN_TRANSCODER_GST (data);
   XfburnTranscoderGstPrivate *priv= XFBURN_TRANSCODER_GST_GET_PRIVATE (trans);
@@ -658,7 +517,7 @@ on_pad_added (GstElement *element, GstPad *pad, gboolean last, gpointer data)
   GstCaps *caps;
   GstStructure *str;
   GstPad *audiopad;
-  GstElement *audio = (GstElement *) priv->resample;
+  GstElement *audio = (GstElement *) priv->conv1;
 
   // only link once
   audiopad = gst_element_get_static_pad (audio, "sink");
@@ -673,26 +532,25 @@ on_pad_added (GstElement *element, GstPad *pad, gboolean last, gpointer data)
 #endif
 
   // check disc type
-  caps = gst_pad_get_caps (pad);
+  caps = gst_pad_query_caps (pad, NULL);
   str = gst_caps_get_structure (caps, 0);
   if (!g_strrstr (gst_structure_get_name (str), "audio")) {
     GstStructure *msg_struct;
     GstMessage *msg;
     GstBus *bus;
 
-    gchar *error_msg = "File content has a decoder but is not audio.";
+    const gchar *error_msg = N_("File content has a decoder but is not audio.");
 
     DBG ("%s", error_msg);
     
     gst_caps_unref (caps);
     gst_object_unref (audiopad);
 
-    priv->is_audio = FALSE;
     g_set_error (&(priv->error), XFBURN_ERROR, XFBURN_ERROR_GST_NO_AUDIO,
 		    "%s",
                  _(error_msg));
     
-    msg_struct = gst_structure_new ("no-audio-content", NULL);
+    msg_struct = gst_structure_new_empty ("no-audio-content");
 
     msg = gst_message_new_application (GST_OBJECT (element), msg_struct);
 
@@ -741,7 +599,7 @@ get_description (XfburnTranscoder *trans)
 static void
 cb_handoff (GstElement *element, GstBuffer *buffer, gpointer data)
 {
-  guint size = GST_BUFFER_SIZE (buffer);
+  guint size = gst_buffer_get_size (buffer);
 #if DEBUG_GST > 2
   static int i = 0;
   const int step = 30;
@@ -756,6 +614,26 @@ cb_handoff (GstElement *element, GstBuffer *buffer, gpointer data)
 
 #endif
 
+static gchar *
+get_discoverer_required_plugins_message (GstDiscovererInfo *info)
+{
+  GString *str;
+  gchar **plugins;
+  gchar *plugins_str;
+
+  plugins = (gchar **) gst_discoverer_info_get_missing_elements_installer_details (info);
+
+  if (g_strv_length(plugins) == 0) {
+    str = g_string_new (_("No information available on which plugin is required."));
+  } else {
+    str = g_string_new (_("Required plugins: "));
+    plugins_str = g_strjoinv (", ", plugins);
+    g_string_append (str, plugins_str);
+    g_free (plugins_str);
+  }
+
+  return g_string_free (str, FALSE);
+}
 
 static gboolean
 get_audio_track (XfburnTranscoder *trans, XfburnAudioTrack *atrack, GError **error)
@@ -764,46 +642,62 @@ get_audio_track (XfburnTranscoder *trans, XfburnAudioTrack *atrack, GError **err
   XfburnTranscoderGstPrivate *priv= XFBURN_TRANSCODER_GST_GET_PRIVATE (tgst);
 
   XfburnAudioTrackGst *gtrack;
-  GTimeVal tv;
   off_t size;
+  GstDiscovererInfo *info;
+  GstDiscovererResult result;
+  gchar *uri;
 
-  priv->is_audio = FALSE;
 #if DEBUG_GST > 0
   DBG ("Querying GST about %s", atrack->inputfile);
 #endif
 
-  priv->state = XFBURN_TRANSCODER_GST_STATE_IDENTIFYING;
-  g_object_set (G_OBJECT (priv->source), "location", atrack->inputfile, NULL);
-  if (gst_element_set_state (priv->pipeline, GST_STATE_PAUSED) == GST_STATE_CHANGE_FAILURE) {
-#if DEBUG_GST > 0
-    g_message ("Supposedly failed to change gstreamer state, ignoring it as usually it does it anyways.");
-#endif
-  }
+  uri = g_strdup_printf("file://%s", atrack->inputfile);
+  info = gst_discoverer_discover_uri(priv->discoverer, uri, error);
+  g_free(uri);
 
-  g_get_current_time (&tv);
-  g_time_val_add (&tv, SIGNAL_WAIT_TIMEOUT_MICROS);
-#if DEBUG_GST > 0
-  DBG ("Now waiting for identification result");
-#endif
-  if (!g_cond_timed_wait (priv->gst_cond, priv->gst_mutex, &tv)) {
-    DBG ("gst identification timed out");
-    recreate_pipeline (tgst);
-    g_set_error (error, XFBURN_ERROR, XFBURN_ERROR_GST_TIMEOUT,
-                 _("Gstreamer did not like this file (detection timed out)"));
-    return FALSE;
-  }
-#if DEBUG_GST > 0
-  DBG ("Got an identification result ");
-#endif
-
-  if (!priv->is_audio) {
-    g_set_error (error, XFBURN_ERROR, XFBURN_ERROR_NOT_AUDIO_FORMAT,
-                 _("%s\n\nis not an audio file:\n\n%s"), atrack->inputfile, priv->error->message);
-    g_error_free (priv->error);
-    priv->error = NULL;
+  if (info == NULL) {
+    if (error && *error == NULL) {
+        DBG ("no info and no error");
+        g_set_error (error, XFBURN_ERROR, XFBURN_ERROR_GST_DISCOVERER,
+                    _("An error occurred while identifying '%s' with gstreamer"), atrack->inputfile);
+    }
     return FALSE;
   }
 
+  result = gst_discoverer_info_get_result(info);
+  switch (result) {
+    case GST_DISCOVERER_MISSING_PLUGINS:
+      g_clear_error (error);
+      /*
+      g_set_error (error, XFBURN_ERROR, XFBURN_ERROR_GST_DISCOVERER,
+                  _("%s: %s"), atrack->inputfile, get_discoverer_required_plugins_message (info));
+      */
+      // the message is pretty useless, so print it only on the console, and create our own instead.
+      DBG ("%s", get_discoverer_required_plugins_message (info));
+      g_set_error (error, XFBURN_ERROR, XFBURN_ERROR_MISSING_PLUGIN,
+                   _(errormsg_missing_plugin),
+                   _("A plugin"));
+
+      gst_discoverer_info_unref(info);
+      return FALSE;
+    case GST_DISCOVERER_OK:
+      break;
+    default:
+      gst_discoverer_info_unref(info);
+      DBG ("gst discoverer said %d", result);
+      /* TODO: improve error messages */
+      //recreate_pipeline (tgst);
+      if (error && *error == NULL)
+        g_set_error (error, XFBURN_ERROR, XFBURN_ERROR_GST_DISCOVERER,
+                     _("An error occurred while identifying '%s' with gstreamer"), atrack->inputfile);
+      else
+        g_prefix_error (error, "%s: ", atrack->inputfile);
+      return FALSE;
+  }
+
+  priv->gst_done = FALSE;
+
+  priv->duration = gst_discoverer_info_get_duration(info);
   atrack->length = priv->duration / 1000000000;
 
   size = (off_t) floorf (priv->duration * (PCM_BYTES_PER_SECS / (float) 1000000000));
@@ -818,6 +712,8 @@ get_audio_track (XfburnTranscoder *trans, XfburnAudioTrack *atrack, GError **err
 
   gtrack->size = size;
 
+  gst_discoverer_info_unref(info);
+
   return TRUE;
 }
 
@@ -900,7 +796,9 @@ prepare (XfburnTranscoder *trans, GError **error)
   XfburnTranscoderGst *gst = XFBURN_TRANSCODER_GST (trans);
   XfburnTranscoderGstPrivate *priv= XFBURN_TRANSCODER_GST_GET_PRIVATE (gst);
   gboolean ret;
-  GTimeVal tv;
+  gint64 end_time;
+
+  g_mutex_lock(&priv->gst_mutex);
 
   priv->tracks = g_slist_reverse (priv->tracks);
 
@@ -908,14 +806,14 @@ prepare (XfburnTranscoder *trans, GError **error)
   ret = transcode_next_track (gst, error);
 
   //DBG ("Waiting for start signal");
-  g_get_current_time (&tv);
-  g_time_val_add (&tv, SIGNAL_WAIT_TIMEOUT_MICROS);
-  if (!g_cond_timed_wait (priv->gst_cond, priv->gst_mutex, &tv)) {
-    recreate_pipeline (gst);
-    g_set_error (error, XFBURN_ERROR, XFBURN_ERROR_GST_TIMEOUT,
-                 _("Gstreamer did not want to start transcoding (timed out)"));
-    return FALSE;
-  }
+  end_time = g_get_monotonic_time () + SIGNAL_WAIT_TIMEOUT_MS * G_TIME_SPAN_MILLISECOND;
+  while (!priv->gst_done)
+    if (!g_cond_wait_until (&priv->gst_cond, &priv->gst_mutex, end_time)) {
+      recreate_pipeline (gst);
+      g_set_error (error, XFBURN_ERROR, XFBURN_ERROR_GST_TIMEOUT,
+                  _("Gstreamer did not want to start transcoding (timed out)"));
+      return FALSE;
+    }
   //DBG ("Got the start signal");
 
   priv->state = XFBURN_TRANSCODER_GST_STATE_TRANSCODING;
@@ -1000,6 +898,8 @@ finish (XfburnTranscoder *trans)
   }
   */
   recreate_pipeline (gst);
+
+  g_mutex_unlock (&priv->gst_mutex);
 }
 
 
